import { defineStore } from 'pinia'
import useFetch from '@/hooks/useFetch'
import { sessionStorage } from '@/plugins/storage'
import type { ExtractApiResponse } from 'api-type'
interface MainStoreState {
  bootstrapTime: number
  loginInfo?: ExtractApiResponse<'userLogin'>
  myUserInfo?: ExtractApiResponse<'getMyUserInfo'>
}

interface MainStoreActions {
  getMyUserInfo: () => Promise<ExtractApiResponse<'getMyUserInfo'> | undefined>
}
const { fetch, loading } = useFetch('getMyUserInfo')

const useMainStore = defineStore<'main', MainStoreState, Record<string, any>, MainStoreActions>('main', {
  state() {
    return {
      /** 进入系统的时间 */
      bootstrapTime: Date.now(),
      /** 用户登录信息 */
      loginInfo: (sessionStorage.get('LOGIN_RESULT') as ExtractApiResponse<'userLogin'>) ?? void 0,
      myUserInfo: void 0,
    }
  },
  actions: {
    async getMyUserInfo() {
      try {
        if (loading.value) {
          return
        }
        this.myUserInfo = await fetch()
        return this.userInfo
      } catch (error) {
        console.warn(error)
      }
    },
  },
})

export default useMainStore
